@@ -1,4 +1,4 @@
-# $Id: Brick.pm 2264 2007-05-09 17:06:24Z comdog $
+# $Id: Brick.pm 2237 2007-03-24 05:00:53Z comdog $
 package Brick;
 use strict;
 
@@ -7,12 +7,9 @@ use vars qw($VERSION);
 
 use Carp qw( carp croak );
 use Data::Dumper;
-use UNIVERSAL qw(isa);
 
-use Brick::Profile;
-
-$VERSION = '0.223';
-	#sprintf "0.%04d_01", q$Revision: 2264 $ =~ m/(\d+)/g;
+$VERSION = '0.220_01';
+#sprintf "0.%04d_01", q$Revision: 2237 $ =~ m/(\d+)/g;
 
 =head1 NAME
 
@@ -26,7 +23,7 @@ Brick - Complex business rule data validation
 		external_packages => [ qw(Foo::Validator Bar::Validator) ]
 		} );
 
-	my $profile = Brick::Profile->new( $brick,
+	my @profile = (
 		[ required  => sub { .... }    => $hash ],
 		[ optional  => optional_fields => $hash ],
 
@@ -35,12 +32,7 @@ Brick - Complex business rule data validation
 		[ outside   => ex_number       => $hash ],
 		);
 
-	my %input_from_app = (
-		name => 'Joe Snuffy',
-		...
-		);
-		
-	my $results = $brick->apply( $profile, \%%input_from_app );
+	my $results = $brick->apply( \@profile, \%input );
 
 =head1 DESCRIPTION
 
@@ -80,7 +72,7 @@ sub _load_external_packages
 	foreach my $package ( @packages )
 		{
 		eval "package $bucket_class; require $package; $package->import";
-		croak "Could not load $package: $@" if $@;
+		carp "Could not load $package: $@" if $@;
 		}
 
 	}
@@ -167,7 +159,7 @@ sub create_bucket
 	{
 	my( $brick, $profile ) = @_;
 
-	unless( 0 == $brick->profile_class->lint( $profile || [] ) ) # zero but true!
+	unless( 0 == $brick->lint( $profile || [] ) ) # zero but true!
 		{
 		croak "Bad profile for create_bucket! Perhaps you need to check it with lint"
 		};
@@ -253,12 +245,12 @@ sub add_validator_packages
 
 Based on the current instance, create another one just like it but not
 connected to it (in effect forking the instance). After the C<clone>
-you can change new instance without affecting the old one. This is
-handy in C<explain>, for instance, where I want a deep copy for a
-moment. At least I think I want a deep copy.
+you can change new instance without affecting the old one. This is handy
+in C<explain>, for instance, where I want a deep copy for a moment. At least
+I think I want a deep copy.
 
-That's the idea. Right now this just returns the same instance. When
-not using a copy breaks, I'll fix that.
+That's the idea. Right now this just returns the instance. When not using
+a copy breaks, I'll fix that.
 
 =cut
 
@@ -269,7 +261,107 @@ sub clone
 	$brick;
 	}
 
-=item explain( PROFILE || PROFILE_ARRAYREF )
+=item lint
+
+Examine the profile and complain about irregularities in format. This
+only checks the format; it does not try to determine if the profile
+works or makes sense. It returns a hash whose key is the index of the
+profile element and whose value is an anonymous hash to indicate what
+had the error:
+
+	format  -   the element is an arrayref
+	name    -   the name is a scalar
+	method  -   is a code ref or can be found in the package
+					$brick->bucket_class returns
+	args    -   the last element is a hash reference
+
+If the profile is not an array reference, C<lint> immediately returns
+undef or the empty list. In scalar context, C<lint> returns 0 for
+format success and the number of errors (so true) for format failures.
+If there is a format error (e.g. an element is not an array ref), it
+immediately returns the number of errors up to that point.
+
+	my $lint = $brick->lint( \@profile );
+
+	print do {
+		if( not defined $lint ) { "Profile must be an array ref\n" }
+		elsif( $lint )          { "Did not validate, had $lint problems" }
+		else                    { "Woo hoo! Everything's good!" }
+		};
+
+In list context, it returns a hash (a list of one element). The result
+will look something like this hash, which has keys for the elements
+that lint thinks are bad, and the values are anonymous hashes with
+keys for the parts that failed:
+
+	%lint = (
+		1 => {
+			method => "Could not find method foo in package",
+			},
+		4 => {
+			args => "Arguments should be a hash ref, but it was a scalar",
+			}
+		);
+
+If you are using C<AUTOLOAD> to generate some of the methods at
+runtime (i.e. after C<lint> has a chance to check for it), use a
+C<can> method to let C<lint> know that it will be available later.
+
+TO DO:
+
+Errors for duplicate names?
+
+=cut
+
+sub lint
+	{
+	my( $brick, $profile ) = @_;
+
+	return unless(
+		eval { $profile->isa( ref [] ) } or
+		UNIVERSAL::isa( $profile, ref [] )
+		);
+
+	my $lint = {};
+
+	foreach my $index ( 0 .. $#$profile )
+		{
+		my $h = $lint->{$index} = {};
+
+		unless( eval { $profile->[$index]->isa( ref [] ) } or
+			UNIVERSAL::isa(  $profile->[$index], ref [] )
+			)
+			{
+			$h->{format} = "Not an array reference!";
+			last;
+			}
+
+		my( $name, $method, $args ) = @{ $profile->[$index] };
+
+		$h->{name} = "Profile name is not a simple scalar!" if ref $name;
+
+		$h->{args} = "Couldn't find method [$method]" unless
+			eval { $method->isa( ref sub {} ) } or
+			UNIVERSAL::isa( $method, sub {} )    or
+			eval { $brick->bucket_class->can( $method ) }; #XXX which class though?
+
+		$h->{args} = "Args is not a hash reference" unless
+			eval { $args->isa( ref {} ) } or
+			UNIVERSAL::isa( $args, ref {} );
+
+		# args needs what?
+
+		delete $lint->{$index} if 0 == keys %{$lint->{$index}};
+		}
+
+#print STDERR Data::Dumper->Dump( [$lint], [qw(lint)] );
+#use Data::Dumper;
+
+	# set errors;
+	wantarray ? %$lint : ( scalar keys %$lint );
+	}
+
+=item explain( PROFILE_ARRAYREF )
 
 Turn the profile into a textual description without applying it to any
 data. This does not add the profile to instance and it does not add
@@ -292,35 +384,24 @@ it in different ways (text output, hash output).
 
 sub explain
 	{
-	my( $brick, $profile_or_array ) = @_;
+	my( $brick, $profile ) = @_;
 
-	my $profile = do {
-		unless( eval { $profile_or_array->isa( $brick->profile_class ) } )
-			{
-			use Brick::Profile;
-			
-			eval "require " . $brick->profile_class;
-			
-			my $profile = $brick->profile_class->new( $brick, $profile_or_array );
+	my $temp_bean = $brick->clone;
 
-			return unless eval { $profile->isa( $brick->profile_class ) };
-			
-			$profile;
-			}
-		else
-			{
-			$profile_or_array;
-			}
-		};
+	if( $temp_bean->lint( $profile ) )
+		{
+		carp "Profile did not validate!";
+		return;
+		}
 
-	my $bucket   = $profile->get_bucket;
-	my $coderefs = $profile->get_coderefs;
-	my $array    = $profile->get_array;
+	my( $bucket, $refs ) = $temp_bean->create_bucket( $profile );
+		#print STDERR Data::Dumper->Dump( [ $bucket ], [qw(bucket)] );
+		#print STDERR Data::Dumper->Dump( [ $refs ], [qw(refs)] );
 
 	my @entries = map {
 		my $e = $bucket->get_from_bucket( $_ );
 		[ map { $e->$_ } qw(get_coderef get_name) ]
-		} @$coderefs;
+		} @$refs;
 
 	#print STDERR Data::Dumper->Dump( [ \@entries ], [qw(entries)] );
 
@@ -354,142 +435,73 @@ sub explain
 	$str;
 	}
 	
-=item apply(  PROFILE_ARRAYREF | PROFILE OBJECT, INPUT_DATA_HASHREF )
+=item apply(  PROFILE_ARRAYREF, INPUT_DATA_HASHREF )
 
-Apply the profile to the data in the input hash reference. The profile
-can either be a profile object or an array ref that apply() will use to
-create the profile object.
-
-This returns a results object blessed into the class name returned by
-results_class(), which is Brick::Result by default. If you don't like
-that, you can override it in your own subclass.
+Apply the profile to the data in the input hash reference. It returns an
+array reference whose elements correspond to the elements in the profile.
 
 =cut
 
 sub apply
 	{
-	my( $brick, $profile_or_array, $input ) = @_;
+	my( $brick, $profile, $input ) = @_;
+
+	my( $bucket, $refs ) = $brick->create_bucket( $profile );
 
-	my $profile = do {
-		unless( eval { $profile_or_array->isa( $brick->profile_class ) } )
-			{
-			use Brick::Profile;
-			
-			eval "require " . $brick->profile_class;
-			
-			my $profile = $brick->profile_class->new( $brick, $profile_or_array );
-						
-			return unless eval { $profile->isa( $brick->profile_class ) };
-			
-			$profile;
-			}
-		else
-			{
-			$profile_or_array;
-			}
-		};
-	
-	my $bucket   = $profile->get_bucket;
-	my $coderefs = $profile->get_coderefs;
-	my $array    = $profile->get_array;
-	
 	my @entries = map {
 		my $e = $bucket->get_from_bucket( $_ );
 		[ map { $e->$_ } qw(get_coderef get_name) ]
-		} @$coderefs;
+		} @$refs;
 
 	my @results = ();
 
 	foreach my $index ( 0 .. $#entries )
 		{
 		my $e    = $entries[$index];
-		my $name = $array->[$index][0];
-		
-		my $bucket_entry = $bucket->get_from_bucket( "$e->[0]" );
-		my $sub_name     = $bucket_entry->get_name;
-		
+		my $name = $profile->[$index][0];
+
+		print STDERR "Checking $name...\n" if $ENV{DEBUG};
+
 		my $result = eval{ $e->[0]->( $input ) };
 		my $eval_error = $@;
-		
-		carp "Brick: $sub_name: eval error \$\@ is not a string or hash reference"
-			unless( ! ref $eval_error or UNIVERSAL::isa( $eval_error, ref {} ) );
 
-		if( defined $eval_error and isa( $eval_error, ref {} ) )
-			{
-			$result = 0;
-			carp "Brick: $sub_name died with reference, but didn't define 'handler' key"
-				unless exists $eval_error->{handler};
-				
-			carp "Brick: $sub_name died with reference, but didn't define 'message' key"
-				unless exists $eval_error->{message};	
-			}
-		elsif( defined $eval_error ) # but not a reference
-			{
-			$eval_error = {
-				handler       => 'program_error',
-				message       => $eval_error,
-				program_error => 1,
-				errors        => [],
-				};
-			}
-			
-		my $handler = $array->[$index][1];
+		#print STDERR Data::Dumper->Dump( [$eval_error], [qw(eval_error)] );
 
-		my $result_item = $brick->result_class->result_item_class->new(
-			label    => $name,
-			method   => $handler,
-			result   => $result,
-			messages => $eval_error,
-			);
-			
-		push @results, $result_item;
+		$result = 0 if ref $eval_error;
+
+		my $handler = $profile->[$index][1];
+
+		push @results, [ $name, $handler, $result, $@ ];
+		}
+
+	eval { eval "use " . $brick->result_class };
+	if( $@ ) 
+		{ 
+		carp "Couldn not load Result class " . $brick->result_class . "\n" 
 		}
 	
 	return bless \@results, $brick->result_class;
 	}
-		
+	
 =item bucket_class
 
-The namespace where the constraint building blocks are defined. By
-default this is C<Brick::Bucket>. If you don't like that, override
-this in a subclass. Things that need to work with the bucket class
-name, such as a factory method, will use the return value of this
-method.
-
-This method also loads the right class, so if you override it, 
-remember to load the class too!
+The namespace where the constraint building blocks are defined. By default
+this is C<Brick::Bucket>. If you don't like that, override this in a
+subclass.
 
 =cut
 
-sub bucket_class { require Brick::Bucket; 'Brick::Bucket' }
+sub bucket_class { 'Brick::Bucket' }
 
 =item result_class
 
 The namespace that C<apply> uses for its result object. By default
 this is C<Brick::Result>. If you don't like that, override this in a
-subclass. Things that need to work with the result class name, such as
-a factory method, will use the return value of this method.
-
-This method also loads the right class, so if you override it, 
-remember to load the class too!
+subclass.
 
 =cut
 
-sub result_class { require Brick::Result; 'Brick::Result' }
-
-=item profile_class
-
-The namespace for the profile object. By default this is
-C<Brick::Profile>. If you don't like that, override this in a
-subclass. Things that need to work with the result class name, such as
-a factory method, will use the return value of this method.
-
-This method also loads the right class, so if you override it, 
-remember to load the class too!
-
-=cut
-
-sub profile_class { require Brick::Profile; 'Brick::Profile' }
+sub result_class { 'Brick::Result' }
 
 =back
 
